==> smooch_plugin-2.0.31.tar.gz: ios/Classes/SmoochHandler.swift <==
//
// Created by João Paulo Ros on 06/12/17.
// Copyright (c) 2017 PremierSoft Solucoes Mobile. All rights reserved.
//

import Foundation
import Smooch
import Flutter

class NavigationController: UINavigationController {
    
    override open var supportedInterfaceOrientations: UIInterfaceOrientationMask {
        return .portrait
    }
}


class SmoochHandler: NSObject {
    private static let staticRef = SmoochHandler()

    var conversation: SKTConversation? {
        didSet {
            self.conversation?.delegate = self
        }
    }
    var navController = NavigationController()
    
    var smooch_token: String?
    var app_id: String?
    var smoochInited = false
    var properties : [String: Any]?
    var channel: FlutterMethodChannel?

    var menuItensAllowed: Array<String> = []
    var chatColor: UIColor?

    static func shared() -> SmoochHandler {
        return staticRef
    }

    func setMenuItemsAllowed(_ itemsAllowed: Any?) {
        guard let itemsAllowed = itemsAllowed as? Array<String> else {
            menuItensAllowed = []
            return
        }
        menuItensAllowed = itemsAllowed
    }

    func setChatColor(_ hexColor: Any?) {
        guard let hexColor = hexColor as? String else {
            self.chatColor = nil
            return
        }
        self.chatColor = UIColor(hex: hexColor)
    }

    func show(_ channel:FlutterMethodChannel?, progress: @escaping (Int) -> Void) {
        self.channel = channel
        UNUserNotificationCenter.current().removeAllDeliveredNotifications()
        if smoochInited {
            doShow()
            progress(0)
        } else {
            let start = Date().timeIntervalSinceReferenceDate
            DispatchQueue.global(qos: .background).async {
                var inited = false
                while(Date().timeIntervalSinceReferenceDate - start < 30) {
                    usleep(250000)
                    if self.smoochInited {
                        DispatchQueue.main.async {
                            self.doShow()
                            progress(0)
                            inited = true
                        }
                        if inited {
                            break
                        }
                    }
                }
                if !inited {
                    progress(-1)
                }
            }
        }
    }

    private func doShow() {

        let vc: UIViewController? = Smooch.newConversationViewController()

        let backTitle = NSLocalizedString("navigationBackButtonTitle", tableName: "SmoochLocalizable", bundle: Bundle.main, value: "", comment: "")

        let backButton = UIBarButtonItem(
            title: backTitle,
            style: .plain,
            target: self,
            action: #selector(self.dismiss)
        )
        vc?.navigationItem.leftBarButtonItem = backButton

        let helpButton = UIBarButtonItem(
            image: ImagesUtils.helpButtonImage,
            style: .plain,
            target: self,
            action: #selector(helpButtonAction)
        )
        helpButton.tintColor = UIColor(hex: "FF9800")
        vc?.navigationItem.rightBarButtonItem = helpButton

        navController = NavigationController(rootViewController: vc!)
        navController.navigationBar.barTintColor = chatColor
        navController.navigationBar.tintColor = UIColor.white
        navController.navigationBar.titleTextAttributes = [
            NSAttributedStringKey.foregroundColor: UIColor.white,
        ]

        UIApplication.shared.delegate?.window??.rootViewController?.present(self.navController, animated: true, completion:  nil)

        if let properties = properties {
            SKTUser.current()?.addProperties(properties)
        }

        self.checkSession()
    }

    func checkSession() {
        channel?.invokeMethod("shouldSendStart", arguments: nil) { result in
            guard let conversation = self.conversation else {
                NSLog("Smooch Conversarion: NULL")
                return
            }
            NSLog("Smooch Conversarion: Start Message")
            conversation.sendMessage(SKTMessage(text: "iniciar"))
        }
    }

    func closeConversarionViewController() {
        dismiss()
    }

    @objc private func dismiss() {
        UIApplication.shared.delegate?.window??.rootViewController?.dismiss(animated: true, completion: nil)
    }

    @objc private func helpButtonAction() {
        guard let flutterChannel = channel else {
            return
        }
        flutterChannel.invokeMethod("onHelpClick", arguments: nil)
    }
}


// MARK: - Smooch Web Services
extension SmoochHandler {

    func logout(onComplete: @escaping (Int) -> Void) {
        Smooch.logout { (error, userInfo) in
            if let error = error {
                NSLog("Smooch::Logout: %@", error.localizedDescription)
            }
            if let userInfo = userInfo {
                NSLog("Smooch::Logout: %@", userInfo)
            }
            self.conversation = nil
            onComplete(0)
        }
    }
}

// MARK: - Smooch Push Notification
extension SmoochHandler {

    func registerPushToken(pushToken: Data) {
        Smooch.setPushToken(pushToken)
    }

    static func handlePushNotification(userInfo: [AnyHashable: Any]) {
        Smooch.handlePushNotification(userInfo)
    }

}

// MARK: - SKTConversationDelegate
extension SmoochHandler: SKTConversationDelegate {

    func conversation(_ conversation: SKTConversation, unreadCountDidChange unreadCount: UInt) {
        channel?.invokeMethod("updateChatUnreadCount", arguments: unreadCount)
    }

    func conversation(_ conversation: SKTConversation, shouldShowInAppNotificationFor message: SKTMessage) -> Bool {
        return true
    }

    func conversation(_ conversation: SKTConversation, willDisplay message: SKTMessage) -> SKTMessage? {

        let messagesKeyList = [
            "iniciar",
            "session",
            "token:",
            "token2:",
        ]

        if let text = message.text, messagesKeyList.contains(where: text.contains) {
            return nil
        }
        return message
    }

    func conversation(_ conversation: SKTConversation, shouldHandle action: SKTMessageAction) -> Bool {

        guard let actionURL = action.uri else {
            return true
        }

        if action.type != SKTMessageActionTypeWebview {
            return true
        }

        guard let channel = channel else {
            return true
        }

        let methodName: String = "handleMessageAction"
        channel.invokeMethod(methodName, arguments: actionURL.absoluteString, result: nil)
        return false
    }

    func conversation(_ conversation: SKTConversation, didShow viewController: UIViewController) {
        self.checkSession()
    }

}

extension UIColor {

    convenience init(hex: String) {

        let hexString: String = hex.trimmingCharacters(in: CharacterSet.whitespacesAndNewlines)

        let scanner = Scanner(string: hexString)

        if hexString.hasPrefix("#") {
            scanner.scanLocation = 1
        }

        var color: UInt32 = 0
        scanner.scanHexInt32(&color)
        let mask = 0x000000FF
        let r = Int(color >> 16) & mask
        let g = Int(color >> 8) & mask
        let b = Int(color) & mask

        self.init(
            red: CGFloat(r) / 0xff,
            green: CGFloat(g) / 0xff,
            blue: CGFloat(b) / 0xff,
            alpha: 1
        )
    }

}

==> smooch_plugin-2.0.32.tar.gz: ios/Classes/SmoochHandler.swift <==
//
// Created by João Paulo Ros on 06/12/17.
// Copyright (c) 2017 PremierSoft Solucoes Mobile. All rights reserved.
//

import Foundation
import Smooch
import Flutter

class NavigationController: UINavigationController {
    
    override open var supportedInterfaceOrientations: UIInterfaceOrientationMask {
        return .portrait
    }
}


class SmoochHandler: NSObject {
    private static let staticRef = SmoochHandler()

    var conversation: SKTConversation? {
        didSet {
            self.conversation?.delegate = self
        }
    }
    var navController = NavigationController()
    
    var smooch_token: String?
    var app_id: String?
    var smoochInited = false
    var properties : [String: Any]?
    var channel: FlutterMethodChannel?

    var menuItensAllowed: Array<String> = []
    var chatColor: UIColor?

    static func shared() -> SmoochHandler {
        return staticRef
    }

    func setMenuItemsAllowed(_ itemsAllowed: Any?) {
        guard let itemsAllowed = itemsAllowed as? Array<String> else {
            menuItensAllowed = []
            return
        }
        menuItensAllowed = itemsAllowed
    }

    func setChatColor(_ hexColor: Any?) {
        guard let hexColor = hexColor as? String else {
            self.chatColor = nil
            return
        }
        self.chatColor = UIColor(hex: hexColor)
    }

    func show(_ channel:FlutterMethodChannel?, progress: @escaping (Int) -> Void) {
        self.channel = channel
        UNUserNotificationCenter.current().removeAllDeliveredNotifications()
        if smoochInited {
            doShow()
            progress(0)
        } else {
            let start = Date().timeIntervalSinceReferenceDate
            DispatchQueue.global(qos: .background).async {
                var inited = false
                while(Date().timeIntervalSinceReferenceDate - start < 30) {
                    usleep(250000)
                    if self.smoochInited {
                        DispatchQueue.main.async {
                            self.doShow()
                            progress(0)
                            inited = true
                        }
                        if inited {
                            break
                        }
                    }
                }
                if !inited {
                    progress(-1)
                }
            }
        }
    }

    private func doShow() {

        let vc: UIViewController? = Smooch.newConversationViewController()

        let backTitle = NSLocalizedString("navigationBackButtonTitle", tableName: "SmoochLocalizable", bundle: Bundle.main, value: "", comment: "")

        let backButton = UIBarButtonItem(
            title: backTitle,
            style: .plain,
            target: self,
            action: #selector(self.dismiss)
        )
        vc?.navigationItem.leftBarButtonItem = backButton

        let helpButton = UIBarButtonItem(
            image: ImagesUtils.helpButtonImage,
            style: .plain,
            target: self,
            action: #selector(helpButtonAction)
        )
        helpButton.tintColor = UIColor(hex: "FF9800")
        vc?.navigationItem.rightBarButtonItem = helpButton

        navController = NavigationController(rootViewController: vc!)
        navController.navigationBar.barTintColor = chatColor
        navController.navigationBar.tintColor = UIColor.white
        navController.navigationBar.titleTextAttributes = [
            NSAttributedString.Key.foregroundColor: UIColor.white,
        ]

        UIApplication.shared.delegate?.window??.rootViewController?.present(self.navController, animated: true, completion:  nil)

        if let properties = properties {
            SKTUser.current()?.addProperties(properties)
        }

        self.checkSession()
    }

    func checkSession() {
        channel?.invokeMethod("shouldSendStart", arguments: nil) { result in
            guard let conversation = self.conversation else {
                NSLog("Smooch Conversarion: NULL")
                return
            }
            NSLog("Smooch Conversarion: Start Message")
            conversation.sendMessage(SKTMessage(text: "iniciar"))
        }
    }

    func closeConversarionViewController() {
        dismiss()
    }

    @objc private func dismiss() {
        UIApplication.shared.delegate?.window??.rootViewController?.dismiss(animated: true, completion: nil)
    }

    @objc private func helpButtonAction() {
        guard let flutterChannel = channel else {
            return
        }
        flutterChannel.invokeMethod("onHelpClick", arguments: nil)
    }
}


// MARK: - Smooch Web Services
extension SmoochHandler {

    func logout(onComplete: @escaping (Int) -> Void) {
        Smooch.logout { (error, userInfo) in
            if let error = error {
                NSLog("Smooch::Logout: %@", error.localizedDescription)
            }
            if let userInfo = userInfo {
                NSLog("Smooch::Logout: %@", userInfo)
            }
            self.conversation = nil
            onComplete(0)
        }
    }
}

// MARK: - Smooch Push Notification
extension SmoochHandler {

    func registerPushToken(pushToken: Data) {
        Smooch.setPushToken(pushToken)
    }

    static func handlePushNotification(userInfo: [AnyHashable: Any]) {
        Smooch.handlePushNotification(userInfo)
    }

}

// MARK: - SKTConversationDelegate
extension SmoochHandler: SKTConversationDelegate {

    func conversation(_ conversation: SKTConversation, unreadCountDidChange unreadCount: UInt) {
        channel?.invokeMethod("updateChatUnreadCount", arguments: unreadCount)
    }

    func conversation(_ conversation: SKTConversation, shouldShowInAppNotificationFor message: SKTMessage) -> Bool {
        return true
    }

    func conversation(_ conversation: SKTConversation, willDisplay message: SKTMessage) -> SKTMessage? {

        let messagesKeyList = [
            "iniciar",
            "session",
            "token:",
            "token2:",
        ]

        if let text = message.text, messagesKeyList.contains(where: text.contains) {
            return nil
        }
        return message
    }

    func conversation(_ conversation: SKTConversation, shouldHandle action: SKTMessageAction) -> Bool {

        guard let actionURL = action.uri else {
            return true
        }

        if action.type != SKTMessageActionTypeWebview {
            return true
        }

        guard let channel = channel else {
            return true
        }

        let methodName: String = "handleMessageAction"
        channel.invokeMethod(methodName, arguments: actionURL.absoluteString, result: nil)
        return false
    }

    func conversation(_ conversation: SKTConversation, didShow viewController: UIViewController) {
        self.checkSession()
    }

}

extension UIColor {

    convenience init(hex: String) {

        let hexString: String = hex.trimmingCharacters(in: CharacterSet.whitespacesAndNewlines)

        let scanner = Scanner(string: hexString)

        if hexString.hasPrefix("#") {
            scanner.scanLocation = 1
        }

        var color: UInt32 = 0
        scanner.scanHexInt32(&color)
        let mask = 0x000000FF
        let r = Int(color >> 16) & mask
        let g = Int(color >> 8) & mask
        let b = Int(color) & mask

        self.init(
            red: CGFloat(r) / 0xff,
            green: CGFloat(g) / 0xff,
            blue: CGFloat(b) / 0xff,
            alpha: 1
        )
    }

}
